local L = AceLibrary("AceLocale-2.2"):new("sRaidFrames")
local Media = LibStub("LibSharedMedia-2.0")
local sRaidFrames = sRaidFrames

BINDING_HEADER_sRaidFrames = "sRaidFrames"
BINDING_NAME_ShowHideRaidWindows = L["Show/Hide Raid Windows"]
BINDING_NAME_ToggleBuffDebuffview = L["Toggle Buff/Debuff view"]
BINDING_NAME_Toggledisplayofonlydispellabledebuffs = L["Toggle display of only dispellable debuffs"]

sRaidFrames.options = {
	type = 'group',
	handler = sRaidFrames,
	args = {
		spacer = {
			type = "header",
			order = 100,
		},
		
		spacer2 = {
			type = "header",
			order = 600,
		},

		behaviour = {
			name = L["Frame Behaviour"],
			type = "header",
			order = 200,
		},

		informational = {
			name = L["Informational"],
			type = "header",
			order = 300,

		},

		appearance = {
			name = L["Appearance"],
			type = "header",
			order = 400,
		},

		buffsdebuffs = {
			name = L["Buffs & Debuffs"],
			type = "header",
			order = 500,
		},

		lock = {
			name = L["Lock"],
			type = "toggle",
			desc = L["Lock/Unlock the raid frames"],
			get = function()
				return sRaidFrames.opt.Locked
			end,
			set = function(value)
				sRaidFrames.opt.Locked = value
				if not value then
					sRaidFrames:S("ShowGroupTitles", true)
					for _,f in pairs(sRaidFrames.groupframes) do
						sRaidFrames:UpdateTitleVisibility(f.header)
					end
				end
			end,
			disabled = InCombatLockdown,
			order = 2,
		},

		show = {
			name = L["Show"],
			type = "toggle",
			desc = L["Show the raid frames"],
			get = function()
				return sRaidFrames.master:IsVisible()
			end,
			set = function(value)
				if value then
					sRaidFrames.master:Show()
				else
					sRaidFrames.master:Hide()
				end
			end,
			hidden = function() return not sRaidFrames.enabled end,
			disabled = InCombatLockdown,
			order = 3,
		},
		
		hide5man = {
			name = "Hide in 5 man raids",
			type = "toggle",
			desc = "Do not show the raid frames in raids with 5 players or less",
			get = function()
				return sRaidFrames.opt.HideIn5Man
			end,
			set = function(value)
				sRaidFrames.opt.HideIn5Man = value
				if value and GetNumRaidMembers() <= 5 then
					sRaidFrames:DisableFrames()
				end
				sRaidFrames:UpdateRoster()
			end,
			disabled = InCombatLockdown,
			order = 4,
		},

		health = {
			name = L["Health text"],
			type = "text",
			desc = L["Set health display type"],
			get = function()
				return sRaidFrames.opt.HealthFormat
			end,
			set = function(value)
				sRaidFrames.opt.HealthFormat = value
			end,
			validate = {["curmax"] = L["Current and max health"], ["deficit"] = L["Health deficit"], ["percent"] = L["Health percentage"], ["current"] = L["Current health"], ["curdeficit"] = L["Current health with deficit"], ["none"] = L["Hide health text"]},
			order = 301,
		},

		debuffs = {
			name = L["Debuffs"],
			type = "group",
			desc = L["Debuff settings"],
			args = {
				blacklist = {
					name = L["Debuff blacklist"],
					type = "group",
					desc = L["Set a list of debuffs to never be displayed"],
					args = {
						add = {
							name = L["Add debuff"],
							type = "text",
							desc = L["Add a debuff"],
							get = false,
							set = function(value)
								if value and value ~= "" and not sRaidFrames.opt.DebuffFilter[value] and not sRaidFrames.opt.DebuffWhitelist[value] then
									sRaidFrames.opt.DebuffFilter[value] = true
									sRaidFrames:chatUpdateDebuffMenu()
								end
							end,
							usage = L["<name of debuff>"],
						},
					},
					disabled = function() return (sRaidFrames.opt.BuffType == "buffs") end,
				},
				whitelist = {
					name = L["Debuff whitelist"],
					type = "group",
					desc = L["Set a list of debuffs to always be displayed"],
					args = {
						add = {
							name = L["Add debuff"],
							type = "text",
							desc = L["Add a debuff"],
							get = false,
							set = function(value)
								if value and value ~= "" and not sRaidFrames.opt.DebuffWhitelist[value] and not sRaidFrames.opt.DebuffFilter[value] then
									sRaidFrames.opt.DebuffWhitelist[value] = true
									sRaidFrames:chatUpdateDebuffMenu()
								end
							end,
							usage = L["<name of debuff>"],
						},
					},
					disabled = function() return not (sRaidFrames.opt.BuffType ~= "buffs" and sRaidFrames.opt.ShowOnlyDispellable) end,
				},

				curable = {
					name = L["Show only curable debuffs"],
					type = "toggle",
					desc = L["Show only debuffs that are curable by me"],
					get = function()
						return sRaidFrames.opt.ShowOnlyDispellable
					end,
					set = "chatToggleDispellable",
					disabled = function() return (sRaidFrames.opt.BuffType == "buffs") end,
				},

				highlight = {
					name = L["Highlight debuffed units"],
					type = "text",
					desc = L["Highlight units afflicted by a certain debuff type"],
					get = function()
						return sRaidFrames.opt.HighlightDebuffs
					end,
					set = function(value)
						sRaidFrames.opt.HighlightDebuffs = value
						sRaidFrames:UpdateAllUnits()
					end,
					validate = {["never"] = L["Never"], ["all"] = L["All dispellable debuffs"], ["onlyself"] = L["Debuffs dispellable by me"]},
				},
			},
			order = 501,
		},

		buffs = {
			name = L["Buffs"],
			type = "group",
			desc = L["Buff settings"],
			args = {
				filter = {
					name = L["Buff filter"],
					type = "group",
					desc = L["Set a list of buffs to be displayed, rather than displaying all castable buffs"],
					args = {
						add = {
							name = L["Add buff"],
							type = "text",
							desc = L["Add a buff"],
							get = false,
							set = function(value)
								if value and value ~= "" and not sRaidFrames.opt.BuffFilter[value] then
									sRaidFrames.opt.BuffFilter[value] = true
									sRaidFrames:chatUpdateBuffMenu()
								end
							end,
							usage = L["<name of buff>"],
						},
					},
					disabled = function() return (sRaidFrames.opt.BuffType == "debuffs") end,
				},
			},
			order = 501,
		},

		titles = {
			name = L["Show group titles"],
			type = "toggle",
			desc = L["Toggle display of titles above each group frame"],
			get = function()
				return sRaidFrames.opt.ShowGroupTitles
			end,
			set = function(value)
				sRaidFrames:S("ShowGroupTitles", value)
				for _,f in pairs(sRaidFrames.groupframes) do
					sRaidFrames:UpdateTitleVisibility(f.header)
				end
			end,
			disabled = function() return not sRaidFrames.opt.Locked or InCombatLockdown() end,
			order = 401,
		},

		bufftype = {
			name = L["Buff/Debuff visibility"],
			type = "text",
			desc = L["Show buffs or debuffs on the raid frames"],
			get = function()
				return sRaidFrames.opt.BuffType
			end,
			set = "chatBuffType",
			validate = {["buffs"] = L["Only buffs"], ["debuffs"] = L["Only debuffs"], ["buffsifnotdebuffed"] = L["Buffs if not debuffed"]},
			order = 501,
		},

		powerfilter = {
			name = L["Power type visiblity"],
			type = "group",
			desc = L["Toggle the display of certain power types (Mana, Rage, Energy)"],
			args = {
				mana = {
					name = L["Mana"],
					type = "toggle",
					desc = L["Toggle the display of mana bars"],
					get = function()
						return sRaidFrames.opt.PowerFilter[0]
					end,
					set = function(value)
						sRaidFrames.opt.PowerFilter[0] = value
						sRaidFrames:UpdateAllUnits()
					end,
					map = {[false] = L["hidden"], [true] = L["shown"]},
				},
				energy = {
					name = L["Energy & Focus"],
					type = "toggle",
					desc = L["Toggle the display of energy and focus bars"],
					get = function()
						return sRaidFrames.opt.PowerFilter[2]
					end,
					set = function(value)
						sRaidFrames.opt.PowerFilter[2] = value
						sRaidFrames.opt.PowerFilter[3] = value
						sRaidFrames:UpdateAllUnits()
					end,
					map = {[false] = L["hidden"], [true] = L["shown"]},
				},
				rage = {
					name = L["Rage"],
					type = "toggle",
					desc = L["Toggle the display of rage bars"],
					get = function()
						return sRaidFrames.opt.PowerFilter[1]
					end,
					set = function(value)
						sRaidFrames.opt.PowerFilter[1] = value
						sRaidFrames:UpdateAllUnits()
					end,
					map = {[false] = L["hidden"], [true] = L["shown"]},
				},
			},
			order = 301,
		},

		invert = {
			name = L["Invert health bars"],
			type = "toggle",
			desc = L["Invert the growth of the health bars"],
			get = function()
				return sRaidFrames.opt.Invert
			end,
			set = function(value)
				sRaidFrames.opt.Invert = value
				sRaidFrames:UpdateAllUnits()
			end,
			order = 201,
		},

		texture = {
			name = L["Bar textures"],
			type = "text",
			desc = L["Set the texture used on health and mana bars"],
			get = function()
				return sRaidFrames.opt.Texture
			end,
			set = "chatTexture",
			validate = Media:List("statusbar"),
			order = 401,
		},

		scale = {
			name = L["Scale"],
			type = "range",
			desc = L["Set the scale of the raid frames"],
			min = 0.1,
			max = 3.0,
			step = 0.05,
			get = function()
				return sRaidFrames.opt.Scale
			end,
			set = "chatScale",
			disabled = InCombatLockdown,
			order = 401,
		},

		positioning = {
			name = L["Positioning"],
			type = "group",
			desc = L["Set the positioning of the raid frames"],
			args = {
				reset = {
					name = L["Reset position"],
					type = "execute",
					desc = L["Reset the position of sRaidFrames"],
					func = "ResetPosition"
				},
				predefined = {
					name = L["Predefined position"],
					type = "text",
					desc = L["Set a predefined position for the raid frames"],
					get = function() return nil end,
					set = "chatSetPosition",
					validate = {["ctra"] = L["CT_RaidAssist"], ["horizontal"] = L["Horizontal"], ["vertical"] = L["Vertical"]},
				},
			},
			disabled = InCombatLockdown,
			order = 201,
		},

		layout = {
			name = L["Layout"],
			type = "text",
			desc = L["Set the layout of the frames based on presets"],
			get = function()
				return sRaidFrames.opt.Layout
			end,
			set = function(value)
				sRaidFrames.opt.Layout = value
				sRaidFrames:SetLayout(value)
				sRaidFrames:ApplyLayout()
			end,
			validate = {},
			order = 201,
		},

		colors = {
			name = L["Colors"],
			type = "group",
			desc = L["Set the diffirent colors of the raid frames"],
			args = {
				background = {
					type = "color",
					name = L["Background color"],
					desc = L["Change the background color"],
					get = function()
						local s = sRaidFrames.opt.BackgroundColor
						return s.r, s.g, s.b, s.a
					end,
					set = "chatBackgroundColor",
					hasAlpha = true,
					order = 100,
				},
				border = {
					type = "color",
					name = L["Border color"],
					desc = L["Change the border color"],
					get = function()
						local s = sRaidFrames.opt.BorderColor
						return s.r, s.g, s.b, s.a
					end,
					set = "chatBorderColor",
					hasAlpha = true,
					disabled = function() return not sRaidFrames.opt.Border end,
					order = 100,
				},

				healthtext = {
					type = "color",
					name = L["Health text color"],
					desc = L["Change the color of the health text"],
					get = function()
						local s = sRaidFrames.opt.HealthTextColor
						return s.r, s.g, s.b, s.a
					end,
					set = "chatHealthTextColor",
					hasAlpha = false,
					order = 100,
				},


				{
					type = "header",
					name = L["Debuff colors"],
					order = 200,
				},

				magic = {
					type = "color",
					name = L["Magic debuff"],
					desc = L["Change the background color for magic debuffs"],
					get = function()
						local s = sRaidFrames.opt.DebuffColors["Magic"]
						return s.r, s.g, s.b, s.a
					end,
					set = function(r, g, b, a)
						sRaidFrames.opt.DebuffColors["Magic"] = {r = r, g = g, b = b, a = a}
					end,
					hasAlpha = true,
					order = 201,
				},
				curse = {
					type = "color",
					name = L["Curse debuff"],
					desc = L["Change the background color for curse debuffs"],
					get = function()
						local s = sRaidFrames.opt.DebuffColors["Curse"]
						return s.r, s.g, s.b, s.a
					end,
					set = function(r, g, b, a)
						sRaidFrames.opt.DebuffColors["Curse"] = {r = r, g = g, b = b, a = a}
					end,
					hasAlpha = true,
					order = 201,
				},
				poison = {
					type = "color",
					name = L["Poison debuff"],
					desc = L["Change the background color for poison debuffs"],
					get = function()
						local s = sRaidFrames.opt.DebuffColors["Poison"]
						return s.r, s.g, s.b, s.a
					end,
					set = function(r, g, b, a)
						sRaidFrames.opt.DebuffColors["Poison"] = {r = r, g = g, b = b, a = a}
					end,
					hasAlpha = true,
					order = 201,
				},
				disease = {
					type = "color",
					name = L["Disease debuff"],
					desc = L["Change the background color for disease debuffs"],
					get = function()
						local s = sRaidFrames.opt.DebuffColors["Disease"]
						return s.r, s.g, s.b, s.a
					end,
					set = function(r, g, b, a)
						sRaidFrames.opt.DebuffColors["Disease"] = {r = r, g = g, b = b, a = a}
					end,
					hasAlpha = true,
					order = 201,
				},
			},
			order = 401,
		},

		tooltips = {
			name = L["Tooltip display"],
			type = "group",
			desc = L["Determine when a tooltip is displayed"],
			args = {
				units = {
					name = L["Unit tooltips"],
					type = "text",
					desc = L["Determine when a tooltip is displayed"],
					get = function() return sRaidFrames.opt.UnitTooltipMethod end,
					set = function(value)
						sRaidFrames:S("UnitTooltipMethod", value)
					end,
					validate = {["never"] = L["Never"], ["notincombat"] = L["Only when not in combat"], ["always"] = L["Always"]},
				},
				buffs = {
					name = L["Buff tooltips"],
					type = "text",
					desc = L["Determine when a tooltip is displayed"],
					get = function() return sRaidFrames.opt.BuffTooltipMethod end,
					set = function(value)
						sRaidFrames:S("BuffTooltipMethod", value)
					end,
					validate = {["never"] = L["Never"], ["notincombat"] = L["Only when not in combat"], ["always"] = L["Always"]},
				},
				debuffs = {
					name = L["Debuff tooltips"],
					type = "text",
					desc = L["Determine when a tooltip is displayed"],
					get = function() return sRaidFrames.opt.DebuffTooltipMethod end,
					set = function(value)
						sRaidFrames:S("DebuffTooltipMethod", value)
					end,
					validate = {["never"] = L["Never"], ["notincombat"] = L["Only when not in combat"], ["always"] = L["Always"]},
				},
			},
			order = 401,
		},

		aggro = {
				name = L["Highlight units with aggro"],
				type = "toggle",
				desc = L["Turn the border of units who have aggro red"],
				get = function() return sRaidFrames.opt.AggroCheck end,
				set = function(value)
					sRaidFrames.opt.AggroCheck = value
				end,
				disabled = function() return not sRaidFrames:GetLayout().HasBorders end,
				order = 301,
		},

		highlight = {
				name = L["Highlight targetted unit"],
				type = "toggle",
				desc = L["Turn the border of unit you are targetting orange"],
				get = function() return sRaidFrames.opt.HighlightTarget end,
				set = function(value)
					sRaidFrames:S("HighlightTarget", value)
				end,
				disabled = function() return not sRaidFrames:GetLayout().HasBorders end,
				order = 301,
		},

		heals = {
			name = L["Heal tracking"],
			type = "group",
			desc = L["Options for heal tracking"],
			disabled = function() return not LibStub:GetLibrary("LibHealComm-3.0", true) end,
			args = {
				enable = {
						name = L["Enable heal tracking"],
						type = "toggle",
						desc = L["Turn the border of units getting heals green"],
						get = function() return sRaidFrames.opt.HighlightHeals end,
						set = function(value)
							sRaidFrames:S("HighlightHeals", value)
						end,

				},
				self = {
						name = L["Show self heals"],
						type = "toggle",
						desc = L["Show heals originating from yourself"],
						get = function() return sRaidFrames.opt.HighlightHealsSelf end,
						set = function(value)
							sRaidFrames:S("HighlightHealsSelf", value)
						end,
						disabled = function() return not sRaidFrames.opt.HighlightHeals end,

				},
				versions = {
						name = "Show users and versions",
						type = "execute",
						desc = "Show a list of users whom are broadcasting heal data",
						func = function() sRaidFrames:ShowHealCommVersions() end,
						order = 200,
				},
			},
			order = 301,
		},

		range = {
			name = L["Range"],
			type = "group",
			desc = L["Options for range checks"],
			args = {
				enable = {
					name = L["Enable range check"],
					type = "toggle",
					desc = L["Enable range checking"],
					get = function() return sRaidFrames.opt.RangeCheck end,
					set = function(value)
						sRaidFrames.opt.RangeCheck = value
						if not value then
							for unit in pairs(sRaidFrames.frames) do
								sRaidFrames.frames[unit]:SetAlpha(1)
							end
						end
					end,
					order = 1,
				},
				alpha = {
					name = L["Alpha"],
					type = "range",
					desc = L["The alpha level for units who are out of range"],
					get = function() return sRaidFrames.opt.RangeAlpha end,
					set = function(value)
						sRaidFrames.opt.RangeAlpha = value
					end,
					min  = 0,
					max  = 1,
					step = 0.1,
					disabled = function() return not sRaidFrames.opt.RangeCheck end,
				},
				frequency = {
					name = L["Frequency"],
					type = "range",
					desc = L["The interval between which range checks are performed"],
					get = function() return sRaidFrames.opt.RangeFrequency end,
					set = function(value)
						sRaidFrames.opt.RangeFrequency = value
						sRaidFrames:UpdateRangeFrequency()
					end,
					min  = 0.2,
					max  = 2.0,
					step = 0.1,
					disabled = function() return not sRaidFrames.opt.RangeCheck end,
				},
				limit = {
					name = L["Range"],
					type = "text",
					desc = L["The range at which a unit is considered out of range"],
					get = function() return tostring(sRaidFrames.opt.RangeLimit) end,
					set = function(value)
						sRaidFrames.opt.RangeLimit = tonumber(value)
					end,
					validate = {},
					disabled = function() return not sRaidFrames.opt.RangeCheck end,
				}
			},
			order = 301,
		},
		
		readycheck = {
			name = L["Display ready check"],
			type = "toggle",
			desc = L["Display readiness status for the raid"],
			get = function() return sRaidFrames.opt.ReadyCheck end,
			set = function(value)
				sRaidFrames.opt.ReadyCheck = value
			end,
			order = 301,
		},

		sets = {
			name = L["Filter/Sort sets"],
			type = "group",
			desc = L["Toggle the display of certain Groups/Classes"],
			args = {},
			disabled = InCombatLockdown,
			order = 201,
		},

		growth = {
			name = L["Growth"],
			type = "text",
			desc = L["Set the growth of the raid frames"],
			get = function()
				return sRaidFrames.opt.Growth
			end,
			set = function(value)
				sRaidFrames.opt.Growth = value
				sRaidFrames:SetGrowth()
			end,
			validate = {["up"] = L["Up"], ["down"] = L["Down"], ["left"] = L["Left"], ["right"] = L["Right"]},
			disabled = InCombatLockdown,
			order = 201,
		},

		spacing = {
			name = L["Frame Spacing"],
			type = "range",
			desc = L["Set the spacing between each of the raid frames"],
			min = -5,
			max = 5,
			step = 1,
			get = function()
				return sRaidFrames.opt.Spacing
			end,
			set = function(s)
				sRaidFrames.opt.Spacing = s
				sRaidFrames:SetSpacing()
			end,
			disabled = InCombatLockdown,
			order = 201,
		},
		
		advanced = {
			name = "Advanced",
			type = "group",
			desc = "Advanced options",
			args = {
				elements = {
					name = "Status elements",
					type = "group",
					desc = "Manage status elements",
					args = {
						
					},
				},	
			},
			order = 601,
		},
	},
}

local validateElements = {["border"] = "Border", ["background"] = "Background", ["statusbar"] = "Statusbar"}

function sRaidFrames:chatUpdateStatusElements()
	local i = 1
	for key, data in pairs(self.opt.StatusMaps) do
		if not self.options.args.advanced.args.elements.args[key] then
			self.options.args.advanced.args.elements.args[key] = {
				type = 'group',
				name = key,
				desc = ("Change settings for the status %s"):format(key),
				args = {
					priority = {
						name = "Priority",
						desc = "Set the priority",
						type = "range",
						min = 1,
						max = 200,
						step = 1,
						set = function(value) self.opt.StatusMaps[key].priority = value end,
						get = function() return self.opt.StatusMaps[key].priority end,
					},
					elements = {
						name = "GUI Elements",
						desc = "Difine which elements this status will use",
						type = "text",
						multiToggle = true,
						validate = validateElements,
						set = function(element, value) self.opt.StatusMaps[key].elements[element] = value end,
						get = function(value) return self.opt.StatusMaps[key].elements[value] end,
					},
					color = {
						name = "Color",
						desc = "Difine which color this status will use",
						type = "color",
						set = function(value) end,
						get = function() 
							local data = self.opt.StatusMaps[key]
							return data.color.r, data.color.g, data.color.b, data.color.a 
						end,
					},
					text = {
						name = "Text",
						desc = "Difine which text this status will show",
						type = "text",
						usage = "<name>",
						set = function(value) self.opt.StatusMaps[key].text = value end,
						get = function() return self.opt.StatusMaps[key].text end,
					},
				},
				order = 200-data.priority,
			}
		end
	end
end

function sRaidFrames:chatUpdateBuffMenu()
	self.options.args.buffs.args.filter.args.remove = {
		type = 'group',
		name = L["Remove buff"],
		desc = L["Remove buffs from the filter"],
		args = {},
	}

	table.sort(self.opt.BuffFilter)

	local i = 1
	for buff in pairs(self.opt.BuffFilter) do
		local buffName = buff -- Odd hack, don't know
		self.options.args.buffs.args.filter.args.remove.args["buff" .. i] = {
			type = 'execute',
			name = buffName,
			desc = L["Remove %s from the buff filter"]:format(buffName),
			func = function()
				self.opt.BuffFilter[buffName] = nil
				self:chatUpdateBuffMenu()
			end
		}
		i = i + 1
	end
end

function sRaidFrames:chatUpdateDebuffMenu()
	self.options.args.debuffs.args.blacklist.args.remove = {
		type = 'group',
		name = L["Remove debuff"],
		desc = L["Remove debuffs from the filter"],
		args = {},
	}

	table.sort(self.opt.DebuffFilter)

	local i = 1
	for debuff in pairs(self.opt.DebuffFilter) do
		local debuffName = debuff -- Odd hack, don't know
		self.options.args.debuffs.args.blacklist.args.remove.args["debuff" .. i] = {
			type = 'execute',
			name = debuffName,
			desc = L["Remove %s from the filter"]:format(debuffName),
			func = function()
				self.opt.DebuffFilter[debuffName] = nil
				self:chatUpdateDebuffMenu()
			end,
		}
		i = i + 1
	end


	self.options.args.debuffs.args.whitelist.args.remove = {
		type = 'group',
		name = L["Remove debuff"],
		desc = L["Remove debuffs from the filter"],
		args = {},
	}

	table.sort(self.opt.DebuffWhitelist)

	local i = 1
	for debuff in pairs(self.opt.DebuffWhitelist) do
		local debuffName = debuff -- Odd hack, don't know
		self.options.args.debuffs.args.whitelist.args.remove.args["debuff" .. i] = {
			type = 'execute',
			name = debuffName,
			desc = L["Remove %s from the filter"]:format(debuffName),
			func = function()
				self.opt.DebuffWhitelist[debuffName] = nil
				self:chatUpdateDebuffMenu()
			end,
		}
		i = i + 1
	end
end

function sRaidFrames:chatBuffType(value)
	self:S("BuffType", value)

	self:UpdateAllUnits()
end

function sRaidFrames:chatToggleDispellable(value)
	self:S("ShowOnlyDispellable", value)

	self:UpdateAllUnits()
end

function sRaidFrames:chatTexture(texture)
	self.opt.Texture = texture

	local tex = Media:Fetch("statusbar", self.opt.Texture)
	for _, f in pairs(self.frames) do
		f.hpbar:SetStatusBarTexture(tex)
		f.mpbar:SetStatusBarTexture(tex)
	end
end

function sRaidFrames:chatScale(t)
	self:S("Scale", t)

	self.master:SetScale(t)
end

function sRaidFrames:chatBackgroundColor(r, g, b, a)
	self.opt.BackgroundColor = {r = r, g = g, b = b, a = a}

	-- Need to do this, since someone might be debuffed, and so will need a diffirent background color
	self:UpdateAllUnits()
end

function sRaidFrames:chatHealthTextColor(r, g, b, a)
	self.opt.HealthTextColor = {r = r, g = g, b = b, a = a}

	for _, f in pairs(self.frames) do
		f.hpbar.text:SetTextColor(r, g, b, a)
	end
end

function sRaidFrames:chatBorderColor(r, g, b, a)
	self:S("BorderColor", {r = r, g = g, b = b, a = a})

	for k,f in pairs(self.frames) do
		f:SetBackdropBorderColor(r, g, b, a)
	end
end

function sRaidFrames:chatSetPosition(layout)
	self:PositionLayout(layout, self.groupframes[1]:GetLeft(), self.groupframes[1]:GetTop()-UIParent:GetTop())
end


function sRaidFrames:UpdateRangeLimitOptions()
	self.options.args.range.args.limit.validate = {}
	for r, _ in pairs(self.RangeChecks) do
		self.options.args.range.args.limit.validate[tostring(r)] = L["%d yards"]:format(r)
	end
end